import arcgisscripting, os, arcpy, string, math, time
import subprocess
from arcpy import env
from math import *

##gp=arcgisscripting.create(9.3)
env.overwriteOutput="True"

#check if ArcInfo licence vailable
if arcpy.ProductInfo() == "ArcInfo":
    #path to the usmerc tool directory
    usmerc_dir=arcpy.GetParameterAsText(0)
    ua_raster=arcpy.GetParameterAsText(1)
    urban_code="Value"
    HoP=arcpy.GetParameterAsText(2)
    out_dir=arcpy.GetParameterAsText(3)


    
    try:
        report_file=out_dir+"\\"+"step1_working_report.txt"
        s=open(report_file,'w')
        s.write("Build-up raster file: "+ua_raster+"\n")
        
        
        s.write("Horizon of perception is "+str(HoP)+" m"+"\n")

        s.write("--------------------------------------------------"+"\n")
#Preparatory works
        arcpy.AddMessage("The working environment is being prepared")
        arcpy.BuildRasterAttributeTable_management (ua_raster, "Overwrite")
        
        sr = arcpy.Describe(ua_raster).spatialReference
        arcpy.CreateFileGDB_management(out_dir, "work_SI.gdb", "9.3")
        arcpy.CreateFeatureDataset_management(out_dir+"\\work_SI.gdb", "Si_val", sr)
       
        SI_GDB=out_dir+"\\work_SI.gdb"
        
        s.write("Working environment was prepared"+"\n")
              
#Urban point layer creation
        arcpy.AddMessage("The urban layer statistic is being prepared")
        arcpy.MakeRasterLayer_management(ua_raster, "ua_raster_lyr")
        arcpy.SelectLayerByAttribute_management("ua_raster_lyr", "NEW_SELECTION", "VALUE =1")
        arcpy.Frequency_analysis("ua_raster_lyr", out_dir+"\\freq_tbl.dbf", "Value", "Count")
        env.workspace=out_dir
        cur=arcpy.SearchCursor("freq_tbl.dbf")
        row=cur.next()
        while row:
            pocet=str(int(row.getValue("Count")))
            arcpy.AddMessage("Build-up layer contains "+pocet+" build-up points/pixels")                             
            row=cur.next()
                
## Start of the usmerc computation
        start=time.clock()
        ##env.workspace=SI_GDB
        arcpy.AddMessage("------------------------------------------------------------------")
        arcpy.AddMessage("The calculation of the point distances is being started")
        usmerc=usmerc_dir+"\\sivalues.exe"
        arcpy.AddMessage("This process may be time and disk space consuming, depending of the number of urban pixels in the reporting unit, horizon of perception and area of the reporting unit")
        arcpy.AddMessage("Preparing ASCII file from the urban raster")
        out_ascii=out_dir+"\\urban_ascii.asc"
        arcpy.RasterToASCII_conversion(ua_raster, out_ascii)
        HoP1=str(HoP)
        arcpy.AddMessage("Computation in the sivalues.exe tool is being started. Process will run in the separate command window")
        proces=subprocess.Popen([usmerc,"-r", HoP1, "-o", out_ascii])
        proces.wait()
        elapsed=(time.clock()-start)
        hours=elapsed // 3600
        rest=elapsed % 3600
        minutes=rest // 60
        seconds=rest % 60
        arcpy.AddMessage("Computation of Si values is completed")
        s.write("Computation of Si values is completed"+"\n")
        arcpy.AddMessage("Execution time: "+str(int(hours))+" hours "+str(int(minutes))+" minutes "+str(int(seconds))+" seconds")
        s.write("Execution time: "+str(int(hours))+" hours "+str(int(minutes))+" minutes "+str(int(seconds))+" seconds"+"\n")
        arcpy.AddMessage("------------------------------------------------------------------")
        arcpy.AddMessage("The creation of point Si values layer is being started")
        work_raster=out_dir+"\\work_si_raster.tif"
        out_ascii2=out_dir+"\\"+os.path.basename (out_ascii)[:-4]+"_CellResults.asc"
        arcpy.ASCIIToRaster_conversion(out_ascii2, work_raster, "FLOAT")
        arcpy.Delete_management (out_ascii)
        out_raster=out_dir+"\\si_raster.tif"
        arcpy.Clip_management(work_raster, "#", out_raster, "#", "0")
        arcpy.DefineProjection_management(out_raster, sr)
        arcpy.Delete_management (work_raster)
        si_layer=SI_GDB+"\\Si_val\\Si_values"
        arcpy.RasterToPoint_conversion(out_raster, si_layer, "Value")
        arcpy.Delete_management (out_raster)
        arcpy.Delete_management (out_dir+"\\freq_tbl.dbf")
             
        arcpy.AddMessage("Points with joined Si value are stored in the "+si_layer+" layer")
        s.write("Points with joined Si value are stored in the "+si_layer+" layer"+"\n")
        arcpy.AddMessage("This layer is input for next step - sprawl metrics computation")
        arcpy.AddMessage("Next step - run 2_Sprawl metric tool")
        s.write("Process of Si values computation is finished")
        s.close()
        
    except:
        errCode=arcpy.GetReturnCode(3)
        if str(errCode) == "10320":
              
            arcpy.AddError("Error generated by Raster to Point tool")
            arcpy.AddMessage("There was not enough free memory or disk space available to complete the operation")
            s.write("Error generated by Raster to Point tool"+"\n")
            s.write("There was not enough free memory or disk space available to complete the operation"+"\n")
            s.close()
        elif str(errCode) == "10427":
            arcpy.AddError("A problem occurred while writing to the output feature class")
            arcpy.AddMessage("Possible solution:")
            arcpy.AddMessage("Ensure that there is enough free disk space at the specified location to accommodate the expected size of the output. If there is not, either choose a different location with more space or increase the available space at the specified location by removing unnecessary data and trying again.")
            s.write("A problem occurred while writing to the output feature class"+"\n")
            s.write("Possible solution:"+"\n")
            s.write("Ensure that there is enough free disk space at the specified location to accommodate the expected size of the output. If there is not, either choose a different location with more space or increase the available space at the specified location by removing unnecessary data and trying again."+"\n")
            s.close()
        elif str(errCode) == "40002":
            arcpy.AddError("Problems with memory - not enough memory")
            arcpy.AddMessage("This error may be due to one of the following conditions:")
            arcpy.AddMessage("- There is not enough virtual memory, or the available memory is fragmented.")
            arcpy.AddMessage("- The page file size is too small.")
            arcpy.AddMessage("- Not enough memory is available in the temporary directory.")
            s.write("Problems with memory - not enough memory"+"\n")
            s.write("This error may be due to one of the following conditions:"+"\n")
            s.write("- There is not enough virtual memory, or the available memory is fragmented."+"\n")
            s.write("- The page file size is too small."+"\n")
            s.write("- Not enough memory is available in the temporary directory."+"\n")
            s.close()
        elif str(errCode) == "10038":
            arcpy.AddError("There was an error in creating output in the workspace (geodatabase or directory).")
            arcpy.AddMessage("Possible solution:")
            arcpy.AddMessage("Check that you have proper write permissions to the output workspace and that there is sufficient available disk space.")
            s.write("There was an error in creating output in the workspace (geodatabase or directory)."+"\n")
            s.write("Possible solution:"+"\n")
            s.write("Check that you have proper write permissions to the output workspace and that there is sufficient available disk space."+"\n")
            s.close()
        else:
            arcpy.AddMessage(arcpy.GetMessages(2))
        
else:
    prd=arcpy.ProductInfo()
    arcpy.AddError("Need an ArcInfo licence to run this tool")
    arcpy.AddMessage("Only "+str(prd)+" licence is available")
    s.write("Need an ArcInfo licence to run this tool"+"\n")
    s.write("Only "+str(prd)+" licence is available"+"\n")
    s.close()

        
